unit CLSDisasm;

//******************************************************************************
// Esta Unit implementa un completo desensamblador
//******************************************************************************
// Utiliza el engine HDE Disassembler
//******************************************************************************
//  Hacker Disassembler Engine 32 0.11                                        **
//  Author:    Veacheslav Patkov                                              **
//  E-mail:    patkov-mail@mail.ru                                            **
//  Web-site:  http://patkov-site.narod.ru/eng.html                           **
//  Copyright (c) 2006-2008, Veacheslav Patkov                                **
//  All rights reserved.                                                      **
//                                                                            **
//******************************************************************************
// Adaptada a Delphi por:
//******************************************************************************
//                                                                            **
//	Title:		HDEngine 0.11                                                   **
//	Version:	0.11.1                                                          **
//	Desc:		Hacker Disassembler Engine 0.11 (Pascal/Delphi version)           **
//	Author:		in4matics - in4matics [at] hotmail [d0t] com                    **
//	Www:		AT4RE Team Forum - http://www.at4re.com                           **
//	Date:		2008-03-10                                                        **
//******************************************************************************
// La funcin principal de la Unit es:
// DisAsmCLS((A: dword; I: HDEInstruction): String;
// Parmetros:
//        A: DWORD. Contiene la direccin de la instruccin,
//                  Se usa para saltos relativos
//        I: HDEInstruction. Estructura que contiene la decodificacin de la
//                  instruccin.
//                  Ver "HDEngine.pas" para referencia y autores
//
//******************************************************************************
//                                                                            **
//	Title:		CLSDisasm                                                       **
//	Version:	0..1                                                            **
//	Desc:		Desensamblador para 80X86                                         **
//	Author:	 stzwei                                                           **
//	Mail:   stzwei@yahoo.com.ar                                               **
//	Date:		2008-04-19                                                        **
//                                                                            **
//******************************************************************************

interface

uses Windows, SysUtils, HDEngine;

function DisAsmCLS(A: dword; I: HDEInstruction): String;

implementation

const
  Reg32Strs : array[0..7] of string = ('EAX', 'ECX', 'EDX', 'EBX', 'ESP', 'EBP', 'ESI', 'EDI');
  Reg16Strs : array[0..7] of string = ('AX','CX','DX','BX','SP','BP','SI','DI');
  Reg8Strs  : array[0..7] of string = ('AL','CL','DL','BL','AH','CH','DH','BH');
  RegSegStrs: array[0..7] of String = ('ES', 'CS', 'SS', 'DS', 'FS', 'GS', 'SEG?', 'SEG?');
  RegCRStrs : array[0..7] of String = ('CR0', 'CR1', 'CR2', 'CR3', 'CR4', 'CR5', 'CR6', 'CR7');
  RegDRStrs : array[0..7] of String = ('DR0', 'DR1', 'DR2', 'DR3', 'DR4', 'DR5', 'DR6', 'DR7');
  RegTRStrs : array[0..7] of String = ('TR0', 'TR1', 'TR2', 'TR3', 'TR4', 'TR5', 'TR6', 'TR7');
  CondStrs  : array[0..15] of String = ('O', 'NO' , 'B', 'NB', 'E', 'NE', 'BE', 'A', 'S', 'NS', 'PE', 'PO', 'L', 'GE', 'LE', 'G');



function QuitaCerosIzq(S: String): String;
//******************************************************************************
var
  sTemp: String;
  i: Integer;
begin
  if S= '0000000000' then Result := '0'
  else
    for i:= 1 to length(S) do
      if S[i] <> '0' then
      begin
        sTemp:= Copy(S,i,Length(S));
        if sTemp = '' then sTemp:= '0';
        Result:= sTemp;
        break;
      end;
end;

function ReemplazaSeg(S: String; C: Char): String;
//******************************************************************************
begin
 begin
    if  pos('DS:',S) <> 0 then S[pos('DS:',S)]:= C
    else if  pos('ES:',S) <> 0 then S[pos('ES:',S)]:= C
    else if  pos('SS:',S) <> 0 then S[pos('SS:',S)]:= C
    else if  pos('FS:',S) <> 0 then S[pos('SS:',S)]:= C
    else if  pos('GS:',S) <> 0 then S[pos('SS:',S)]:= C;
  end;
  Result:= S;
end;

function signo8(si: byte): String;
//******************************************************************************
begin
  if si > $7F then Result := '-'+IntToHex($100-si,2)
  else Result:= '+'+IntToHex(si,2);
end;


function signo32(dwi: dword): String;
//******************************************************************************
begin
  if dwi > $FFFFC100 then Result := '-'+QuitaCerosIzq(IntToHex($100000000-dwi,10))
  else Result:= '+'+QuitaCerosIzq(IntToHex(dwi,10));
end;


function DirMemReg(I: HDEInstruction; AsmText: String; alig: String; sReg: String): String;
//******************************************************************************
begin
  case I.modrm of
// Mem,Reg
//************
    $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$23, $26..$2B, $2E..$33, $36..$3B, $3E, $3F:
      Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + '],' + sReg;
    $04, $0C, $14, $1C, $24, $2C, $34, $3C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'],' + sReg;
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'],' + sReg;
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_index] + signo32(I.disp32) + '],' + sReg;
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + '],' + sReg;
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + '],' + sReg;
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + sReg;
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E, $5F, $68..$6B, $6E..$73, $76..$7B, $7E..$7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2],' + sReg;
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2],' + sReg;
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*2' + signo32(I.disp32) + '],' + sReg;
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4],' + sReg;
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4],' + sReg;
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*4' + signo32(I.disp32) + '],' + sReg;
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE..$FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8],' + sReg;
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8],' + sReg;
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*8' + signo32(I.disp32) + '],' + sReg;
      end;
    end;
    $05, $0D, $15, $1D, $25, $2D, $35, $3D:
       Result := AsmText + ' ' + alig + ' DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + sReg;
    $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$63, $66..$6B, $6E..$73, $76..$7B, $7E, $7F:
      Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + '],' + sReg;
    $44, $4C, $54, $5C, $64, $6C, $74, $7C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) +'],' + sReg;
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) +'],' + sReg;
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + '],' + sReg;
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + '],' + sReg;
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + '],' + sReg;
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + '],' + sReg;
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E, $5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + '],' + sReg;
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + '],' + sReg;
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*2' + signo8(I.disp8) + '],' + sReg;
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + '],' + sReg;
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + '],' + sReg;
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+'  +Reg32Strs[I.sib_index] + '*4' + signo8(I.disp8) + '],' + sReg;
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + '],' + sReg;
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + '],' + sReg;
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*8' + signo8(I.disp8) + '],' + sReg;
      end;
    end;
    $45, $4D, $55, $5D, $65, $6D, $75, $7D:
      Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + '],' + sReg;
    $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$A3, $A6..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
      Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + signo32(I.disp32)+ '],' + sReg;
    $84, $8C, $94, $9C, $A4, $AC, $B4, $BC:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo32(I.disp32)  +'],' + sReg;
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo32(I.disp32)  +'],' + sReg;
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo32(I.disp32) + '],' + sReg;    //
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + signo32(I.disp32) + '],' + sReg;
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + '],' + sReg;
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' DS:[' + signo32(I.disp32) + '],' + Reg32Strs[I.modrm_reg];
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E, $5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo32(I.disp32) + '],' + sReg;
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo32(I.disp32) + '],' + sReg;
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*2' + signo32(I.disp32) + '],' + sReg;    //
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo32(I.disp32) + '],' + sReg;
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo32(I.disp32) + '],' + sReg;
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*4' + signo32(I.disp32) + '],' + sReg;
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo32(I.disp32) + '],' + sReg;
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo32(I.disp32) + '],' + sReg;
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*8' + signo32(I.disp32) + '],' + sReg;
      end;
    end;
    $85, $8D, $95, $9D, $A5, $AD, $B5, $BD:
      Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + signo32(I.disp32)+ '],' + sReg;

// Reg32,Reg32
//************
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_rm] + ',' + sReg;
   end;
end;


function DirMemReg8_Reg8Reg8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Mem,Reg8
//************
  Result:= DirMemReg(I, AsmText, alig, Reg8Strs[I.modrm_reg]);
// Reg8,Reg8
//**********
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg8Strs[I.modrm_rm] + ',' + Reg8Strs[I.modrm_reg];
  end;
end;

//OK
function DirMemReg32_Reg32Reg32(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Mem,Reg32
//************
  Result := DirMemReg(I, AsmText, alig, Reg32Strs[I.modrm_reg]);
// Reg32,Reg32
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_rm] + ',' + Reg32Strs[I.modrm_reg];
   end;
end;


function DirMemReg16_Reg16Reg16(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Mem,Reg16
//************
  Result:= DirMemReg(I, AsmText, alig, Reg16Strs[I.modrm_reg]);
// Mem,Reg16
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg16Strs[I.modrm_rm] + ',' + Reg16Strs[I.modrm_reg];
  end;
end;


function DirRegMem(I: HDEInstruction; AsmText: String; alig: String; sReg: String): String;
//******************************************************************************
begin
  case I.modrm of
// Reg,Mem
//************
    $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$23, $26..$2B, $2E..$33, $36..$3B, $3E, $3F:
      Result := AsmText + ' '+ sReg +','+ alig + ' DS:[' + Reg32Strs[I.modrm_rm] + ']';
    $04, $0C, $14, $1C, $24, $2C, $34, $3C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' ' + sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +']';
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +']';
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' '+ sReg + ','  + alig + ' DS:[' + Reg32Strs[I.sib_index] + signo32(I.disp32) + ']';
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' '+ sReg +','  + alig + ' DS:[' +Reg32Strs[I.sib_base] + ']';
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + sReg + ','  + alig + ' SS:[' +Reg32Strs[I.sib_base] + ']';
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + sReg + ','  + alig + ' DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E, $5F, $68..$6B, $6E..$73, $76..$7B, $7E..$7F:
            Result := AsmText + ' ' + sReg + ','  + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2]';
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2]';
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*2' + signo32(I.disp32) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4]';
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4]';
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*4' + signo32(I.disp32) + ']';
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE..$FF:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8]';
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8]';
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*8' + signo32(I.disp32) + ']';
      end;
    end;
    $05, $0D, $15, $1D, $25, $2D, $35, $3D:
       Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
    $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$63, $66..$6B, $6E..$73, $76..$7B, $7E, $7F:
      Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + ']';
    $44, $4C, $54, $5C, $64, $6C, $74, $7C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) +']';
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) +']';
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + ']';
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + ']';
        $24, $64, $A4, $E4:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + ']';
        $25, $65, $A5, $E5:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E, $5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + ']';
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + ']';
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*2' + signo8(I.disp8) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + ']';
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + ']';
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+'  +Reg32Strs[I.sib_index] + '*4' + signo8(I.disp8) + ']';
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + ']';
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + ']';
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*8' + signo8(I.disp8) + ']';
      end;
    end;
    $45, $4D, $55, $5D, $65, $6D, $75, $7D:
      Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + ']';
    $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$A3, $A6..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
      Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + signo32(I.disp32)+ ']';
    $84, $8C, $94, $9C, $A4, $AC, $B4, $BC:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo32(I.disp32)  +']';
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo32(I.disp32)  +']';
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo32(I.disp32) + ']';
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' +Reg32Strs[I.sib_base] + signo32(I.disp32) + ']';
        $24, $64, $A4, $E4:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' +Reg32Strs[I.sib_base] + ']';
        $25, $65, $A5, $E5:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + signo32(I.disp32) + ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E, $5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo32(I.disp32) + ']';
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo32(I.disp32) + ']';
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*2' + signo32(I.disp32) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo32(I.disp32) + ']';
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo32(I.disp32) + ']';
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*4' + signo32(I.disp32) + ']';
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' '+ sReg + ',' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo32(I.disp32) + ']';
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo32(I.disp32) + ']';
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' '+sReg + ',' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*8' + signo32(I.disp32) + ']';
      end;
    end;
    $85, $8D, $95, $9D, $A5, $AD, $B5, $BD:
      Result := AsmText + ' '+ sReg + ',' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + signo32(I.disp32)+ ']';
  end;
end;

function DirReg8Mem_Reg8Reg8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Reg8,Mem
//************
  Result:= DirRegMem(I, AsmText, alig, Reg8Strs[I.modrm_reg]);
// Reg8,Reg8
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg8Strs[I.modrm_reg] + ',' + Reg8Strs[I.modrm_rm];
  end;
end;


//OK
function DirReg32Mem_Reg32Reg32(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Reg32,Mem
//************
  Result:= DirRegMem(I, AsmText, alig, Reg32Strs[I.modrm_reg]);
// Reg32,Reg32
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' '+ Reg32Strs[I.modrm_reg] + ',' + Reg32Strs[I.modrm_rm];
  end;
end;



function DirReg32Mem_Reg32Reg8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Reg32,Mem
//************
  Result:= DirRegMem(I, AsmText, alig, Reg32Strs[I.modrm_reg]);
// Reg32,Reg8
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_reg] + ',' + Reg8Strs[I.modrm_rm];
  end;
end;


function DirReg32Mem_Reg32Reg16(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Reg32,Mem
//************
  Result:= DirRegMem(I, AsmText, alig, Reg32Strs[I.modrm_reg]);
// Reg32,Reg16
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_reg] + ',' + Reg16Strs[I.modrm_rm];
  end;
end;


function DirReg16Mem_Reg16Reg16(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
// Reg16,Mem
//************
  Result:= DirRegMem(I, AsmText, alig, Reg16Strs[I.modrm_reg]);
// Reg16,Reg16
//************
  case I.modrm of
    $C0..$FF: Result := AsmText + ' ' + Reg16Strs[I.modrm_reg] + ',' + Reg16Strs[I.modrm_rm];
  end;
end;


function DirAccImm8(I: HDEInstruction; AsmText: String): String;
//******************************************************************************
begin
// Acc,Imm8
//********
  Result:=  AsmText + ' AL,' + IntToHex(I.imm8,2);
end;


function DirAccImm32(I: HDEInstruction; AsmText: String): String;
//******************************************************************************
begin
// Acc,Imm32
//********
//  Result:=  AsmText + ' EAX,' + IntToHex(I.imm32,2);
  Result:=  AsmText + ' EAX,' + IntToHex(I.inm32,2);
end;


function DirAccImm16(I: HDEInstruction; AsmText: String): String;
//******************************************************************************
begin
// Acc,Imm16
//********
  Result:=  AsmText + ' AX,' + IntToHex(I.imm16,2);
end;


function DirMemImm(I: HDEInstruction; AsmText: String; alig: String; ImmAlign: Integer): String;
//******************************************************************************
var
  strImm: String;
begin
{
  if alig = 'DWORD PTR' then strImm:=  QuitaCerosIzq(IntToHex(I.imm32,10))
  else  if alig = 'WORD PTR' then strImm:= IntToHex(I.imm16,4)
  else strImm:= IntToHex(I.imm8,2);
  }
  Case ImmAlign of
//    32: strImm:= QuitaCerosIzq(IntToHex(I.imm32,10));
    32: strImm:= QuitaCerosIzq(IntToHex(I.inm32,10));
    16: strImm:= IntToHex(I.imm16,4);
     8: strImm:= IntToHex(I.imm8,2);
  end;
     {
  if ImmAlign = 32 then strImm:= QuitaCerosIzq(IntToHex(I.imm32,10))
  else if ImmAlign = 16 then strImm:= IntToHex(I.imm16,4)
  else strImm:= IntToHex(I.imm8,2);
  }
  case I.modrm of
    $00..$03, $06..$0B, $0E, $0F,
    $10..$13, $16..$1B, $1E, $1F,
    $20..$23, $26..$2B, $2E, $2F,
    $30..$33, $36..$3B, $3E, $3F:
        Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] +{ '+' + IntToHex(I.disp8,2) +} '],' + strImm;
    $04, $14, $24, $34, $0C, $1C, $2C, $3C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E..$3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +{ '+' + IntToHex(I.disp8,2) +} '],' + strImm;
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +{ '+' + IntToHex(I.disp8,2) +} '],' + strImm;
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' DS:[' +{ Reg32Strs[I.sib_base] + '+' +} Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)){ '+' + IntToHex(I.disp8,2)} + '],' + strImm;
        $20..$23, $26, $27, $60..$63, $66, $67, $A0..$A3, $A6, $A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] +{ '+' + IntToHex(I.disp8,2) +} '],' + strImm;
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] +{ '+' + IntToHex(I.disp8,2) +} '],' + strImm;
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' DS:[' + {Reg32Strs[I.sib_base] + '+' +} QuitaCerosIzq(IntToHex(I.disp32,10)){IntToHex(I.disp8,2)} + '],' + strImm;
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + {'+' + IntToHex(I.disp8,2) +} '],' + strImm;
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + {'+' + IntToHex(I.disp8,2) +} '],' + strImm;
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' DS:[' + {Reg32Strs[I.sib_base] + '+' +} Reg32Strs[I.sib_index] + '*2+' + QuitaCerosIzq(IntToHex(I.disp32,10)){+' + {IntToHex(I.disp8,2)} + '],' + strImm;
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4'{+' + IntToHex(I.disp8,2)} + '],' + strImm;
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4'{+' + IntToHex(I.disp8,2)} + '],' + strImm;
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' DS:[' + {Reg32Strs[I.sib_base] + '+' +} Reg32Strs[I.sib_index] + '*4+' + QuitaCerosIzq(IntToHex(I.disp32,10)){+' + IntToHex(I.disp8,2)} + '],' + strImm;
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE, $DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8'{+' + IntToHex(I.disp8,2)} + '],' + strImm;
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8'{+' + IntToHex(I.disp8,2)} + '],' + strImm;
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' DS:[' + {Reg32Strs[I.sib_base] + '+' +} Reg32Strs[I.sib_index] + '*8+' + QuitaCerosIzq(IntToHex(I.disp32,10)){+' + IntToHex(I.disp8,2)} + '],' + strImm;
      end;
    end;
    $05, $15, $25, $35, $0D, $1D, $2D, $3D:
        Result := AsmText + ' ' + alig + ' DS:[' {+ Reg32Strs[I.modrm_rm] + '+'} + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;//IntToHex(I.imm8,2);
    $40..$43, $46..$4B, $4E, $4F,
    $50..$53, $56..$5B, $5E, $5F,
    $60..$63, $66..$6B, $6E, $6F,
    $70..$73, $76..$7B, $7E, $7F:
        Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + '],' + strImm;
    $44, $54, $64, $74, $4C, $5C, $6C, $7C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E..$3F:
           Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + '],' + strImm;
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + '],' + strImm;
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + '],' + strImm;
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6..$E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + signo8(I.disp8) + '],' + strImm;
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + signo8(I.disp8) + '],' + strImm;
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + signo8(I.disp8) + '],' + strImm;
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$5F, $68..$6B, $6E..$73, $76..$7B, $7E..$7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + '],' + strImm;
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + '],' + strImm;
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*2' + signo8(I.disp8) + '],' + strImm;
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + '],' + strImm;
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4+' + signo8(I.disp8) + '],' + strImm;
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*4' + signo8(I.disp8) + '],' + strImm;
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE..$FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + '],' + strImm;
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + '],' + strImm;
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '*8' + signo8(I.disp8) + '],' + strImm;
      end;
    end;
    $45, $55, $65, $75, $4D, $5D, $6D, $7D:
        Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + '],' + strImm;
    $80..$83, $86..$8B, $8E, $8F,
    $90..$93, $96..$9B, $9E, $9F,
    $A0..$A3, $A6..$AB, $AE, $AF,
    $B0..$B3, $B6..$BB, $BE, $BF:
        Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
    $84, $94, $A4, $B4, $9C, $AC, $BC:
    begin
      case I.sib of
       $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
      end;
    end;
    $85, $95, $A5, $B5, $9D, $AD, $BD:
        Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + strImm;
  end;
end;


function DirMem(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
    $00..$03, $06..$0B, $0E, $0F,
    $10..$13, $16..$1B, $1E, $1F,
    $20..$23, $26..$2B, $2E, $2F,
    $30..$33, $36..$3B, $3E, $3F:
        Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + ']';
    $04, $14, $24, $34, $1C, $2C, $3C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E..$3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +']';
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +']';
        $05, $0D, $15, $1D, $2D, $35, $3D:
          Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6..$E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + ']';
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + ']';
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$5F, $68..$6B, $6E..$73, $76..$7B, $7E..$7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2]';
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2]';
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*2+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4]';
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4]';
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_index] + '*4+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE..$FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8]';
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8]';
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_index] + '*8+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
      end;
    end;
    $05, $15, $25, $35, $1D, $2D, $3D:
        Result := AsmText + ' ' + alig + ' DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
    $40..$43, $46..$4B, $4E, $4F,
    $50..$53, $56..$5B, $5E, $5F,
    $60..$63, $66..$6B, $6E, $6F,
    $70..$73, $76..$7B, $7E, $7F:
        Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + ']';
    $44, $54, $64, $74, $5C, $6C, $7C:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E..$3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + ']';
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + ']';
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + signo8(I.disp8) + ']';
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6..$E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + signo8(I.disp8) + ']';
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + signo8(I.disp8) +  ']';
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' DS:['  +Reg32Strs[I.sib_base] + signo8(I.disp8) +  ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$5F, $68..$6B, $6E..$73, $76..$7B, $7E..$7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + ']';
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + 'SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + ']';
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + 'DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + signo8(I.disp8) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + ']';
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + ']';
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + signo8(I.disp8) + ']';
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE..$FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + ']';
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + ']';
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + signo8(I.disp8) + ']';
      end;
    end;
    $45, $4D, $55, $65, $75, $5D, $6D, $7D:
        Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + signo8(I.disp8) + ']';
    $80..$83, $86..$8B, $8E, $8F,
    $90..$93, $96..$9B, $9E, $9F,
    $A0..$A3, $A6..$AB, $AE, $AF,
    $B0..$B3, $B6..$BB, $BE, $BF:
        Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.modrm_rm] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
    $84, $94, $A4, $B4, $9C, $AC, $BC:
    begin
      case I.sib of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$1F, $28..$2B, $2E..$33, $36..$3B, $3E, $3F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $04, $0C, $14, $1C, $2C, $34, $3C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $05, $0D, $15, $1D, $2D, $35, $3D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6, $E7:
            Result := AsmText + ' ' + alig + ' DS:[' +Reg32Strs[I.sib_base] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $24, $64, $A4, $E4:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $25, $65, $A5, $E5:
            Result := AsmText + ' ' + alig + ' SS:[' +Reg32Strs[I.sib_base] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$5F, $68..$6B, $6E..$73, $76..$7B, $7E, $7F:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $44, $4C, $54, $5C, $6C, $74, $7C:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $45, $4D, $55, $5D, $6D, $75, $7D:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$9F, $A8..$AB, $AE..$B3, $B6..$BB, $BE, $BF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $84, $8C, $94, $9C, $AC, $B4, $BC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $85, $8D, $95, $9D, $AD, $B5, $BD:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $C0..$C3, $C6..$CB, $CE..$D3, $D6..$DB, $DE..$DF, $E8..$EB, $EE..$F3, $F6..$FB, $FE, $FF:
            Result := AsmText + ' ' + alig + ' DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $C4, $CC, $D4, $DC, $EC, $F4, $FC:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $C5, $CD, $D5, $DD, $ED, $F5, $FD:
            Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8' + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
      end;
    end;
    $85, $95, $A5, $B5, $9D, $AD, $BD:
        Result := AsmText + ' ' + alig + ' SS:[' + Reg32Strs[I.modrm_rm] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
  end;
end;


function DirMemImm8_Reg8Imm8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Reg,Imm8
//********
    $00..$BF: Result:= DirMemImm(I, AsmText, alig, 8);
//Reg8,Imm8
//**********
    $C0..$FF: Result := AsmText + ' ' + Reg8Strs[I.modrm_rm] + ',' + IntToHex(I.imm8,2);
  end;
end;


function DirMemImm32_Reg32Imm8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem,Imm32
//********
    $00..$BF: Result:= DirMemImm(I, AsmText, alig, 8);
//Reg32,Imm8
//********
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_rm] + ',' + IntToHex(I.imm8,2);
  end;
end;


function DirMemImm16_Reg16Imm8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem,Imm8
//********
    $00..$BF: Result:= DirMemImm(I, AsmText, alig, 8);
//Reg16,Imm8
//********
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_rm] + ',' + IntToHex(I.imm8,2);
  end;
end;


function DirMemImm32_Reg32Imm32(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem,Imm32
//********
    $00..$BF: Result:= DirMemImm(I, AsmText, alig, 32);
//Reg32,Imm32
//********
//    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_rm] + ',' + QuitaCerosIzq(IntToHex(I.Imm32,10));
    $C0..$FF: Result := AsmText + ' ' + Reg32Strs[I.modrm_rm] + ',' + QuitaCerosIzq(IntToHex(I.Inm32,10));
  end;
end;


function DirMemImm16_Reg16Imm16(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem,Imm16
//*********
    $00..$BF: Result:= DirMemImm(I, AsmText, alig, 16);
//Reg16,Imm16
//********
    $C0..$FF: Result := AsmText + ' ' + Reg16Strs[I.modrm_rm] + ',' + IntToHex(I.imm16,4);
  end;
end;


function DirMem_Reg8(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem
//***
    $00..$BF: Result:= DirMem(I, AsmText, alig);
//Reg8
//****
    $C0..$FF: Result:= AsmText + ' ' + Reg8Strs[I.modrm_rm];
  end;
end;


function DirMem_Reg32(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem
//***
    $00..$BF: Result:= DirMem(I, AsmText, alig);
//Reg32
//****
    $C0..$FF: Result:= AsmText + ' ' + Reg32Strs[I.modrm_rm];
  end;
end;


function DirMem_Reg16(I: HDEInstruction; AsmText: String; alig: String): String;
//******************************************************************************
begin
  case I.modrm of
//Mem
//***
    $00..$BF: Result:= DirMem(I, AsmText, alig);
//Reg16
//****
    $C0..$FF: Result:= AsmText + ' ' + Reg16Strs[I.modrm_rm];
  end;
end;



//******************************************************************************
// Funcion principal de la Unit
// Parmetros:
//        A: DWORD. Contiene la direccin de la instruccin,
//                  Se usa para saltos relativos
//        I: HDEInstruction. Estructura que contiene la decodificacin de la
//                  instruccin.
//                  Ver "HDEngine.pas" para referencia y autores
//
//******************************************************************************
function DisAsmCLS(A: dword; I: HDEInstruction): String;
//******************************************************************************
var
  tmpInt  : Integer;
  tmpStr  : String;
  alig    : String;
  asmText : String;
begin
  tmpStr:= '???';
  tmpInt := I.opcode;
  case tmpInt of
// ADD, MOV
//*********
    $00, $08, $10, $30, $28, $38, $84, $86, $88:
    begin
      alig:= 'BYTE PTR';
      case tmpInt of
        $00: asmText:= 'ADD';
        $08: asmText:= 'OR';
        $10: asmText:= 'ADC';
        $28: asmText:= 'SUB';
        $30: asmText:= 'XOR';
        $38: asmText:= 'CMP';
        $84: asmText:= 'TEST';
        $86: asmText:= 'XCHG';
        $88: asmText:= 'MOV';
      end;
      tmpStr:= DirMemReg8_Reg8Reg8(I, AsmText, alig);
    end;
    $01, $09, $11, $19, $21, $29, $31, $39, $85, $87, $89:
    begin
      case tmpInt of
        $01: asmText:= 'ADD';
        $09: asmText:= 'OR';
        $11: asmText:= 'ADC';
        $19: asmText:= 'SBB';
        $21: asmText:= 'AND';
        $29: asmText:= 'SUB';
        $31: asmText:= 'XOR';
        $39: asmText:= 'CMP';
        $85: asmText:= 'TEST';
        $87: asmText:= 'XCHG';
        $89: asmText:= 'MOV';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr:= DirMemReg32_Reg32Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'WORD PTR';
        tmpStr:= DirMemReg16_Reg16Reg16(I, AsmText, alig);
      end;
    end;
    $02, $0A, $12, $1A, $22, $2A, $32, $3A, $8A:
    begin
      alig:= 'BYTE PTR';
      case tmpInt of
        $02: asmText:= 'ADD';
        $0A: asmText:= 'OR';
        $12: asmText:= 'ADC';
        $1A: asmText:= 'SBB';
        $22: asmText:= 'AND';
        $2A: asmText:= 'SUB';
        $32: asmText:= 'XOR';
        $3A: asmText:= 'CMP';
        $8A: asmText:= 'MOV';
      end;
      tmpStr:= DirReg8Mem_Reg8Reg8(I, AsmText, alig);
    end;
    $03, $0B, $13, $1B, $23, $2B, $33, $3B, $8B:
    begin
      case tmpInt of
        $03: asmText:= 'ADD';
        $0B: asmText:= 'OR';
        $13: asmText:= 'ADC';
        $1B: asmText:= 'SBB';
        $23: asmText:= 'AND';
        $2B: asmText:= 'SUB';
        $33: asmText:= 'XOR';
        $3B: asmText:= 'CMP';
        $8B: asmText:= 'MOV';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr:= DirReg32Mem_Reg32Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'WORD PTR';
        tmpStr:= DirReg16Mem_Reg16Reg16(I, AsmText, alig);
      end;
    end;
    $04, $0C, $14, $1C, $24, $2C, $34, $3C, $A8:
    begin
      case tmpInt of
        $04: asmText:= 'ADD';
        $0C: asmText:= 'OR';
        $14: asmText:= 'ADC';
        $24: asmText:= 'AND';
        $1C: asmText:= 'SBB';
        $2C: asmText:= 'SUB';
        $34: asmText:= 'XOR';
        $3C: asmText:= 'CMP';
        $A8: asmText:= 'TEST';
      end;
      tmpStr:= DirAccImm8(I, AsmText);
    end;
    $05, $0D, $15, $1D, $25, $2D, $35, $3D, $A9:
    begin
      case tmpInt of
        $05: asmText:= 'ADD';
        $0D: asmText:= 'OR';
        $15: asmText:= 'ADC';
        $1D: asmText:= 'SBB';
        $25: asmText:= 'AND';
        $2D: asmText:= 'SUB';
        $35: asmText:= 'XOR';
        $3D: asmText:= 'CMP';
        $A9: asmText:= 'TEST';
      end;
      if I.p_66 = $00 then
        tmpStr:= DirAccImm32(I, AsmText);
      if I.p_66 = $66 then
        tmpStr:= DirAccImm16(I, AsmText);
    end;
    $06: tmpStr := 'PUSH ES';
    $07: tmpStr := 'POP ES';
    $0E: tmpStr := 'PUSH CS';
    $0F:
    begin
      tmpInt := I.opcode2;
      case tmpInt of
        $00:
        begin
          alig:= 'WORD PTR';
          case I.modrm of
            $00..$07, $40..$47, $80..$87, $C0..$C7: AsmText:= 'SLDT';
            $08..$0F, $48..$4F, $88..$8F, $C8..$CF: AsmText:= 'STR';
            $10..$17, $50..$57, $90..$97, $D0..$D7: AsmText:= 'LLDT';
            $18..$1F, $58..$5F, $98..$9F, $D8..$DF: AsmText:= 'LTR';
            $20..$27, $60..$67, $A0..$A7, $E0..$E7: AsmText:= 'VERR';
            $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: AsmText:= 'VERW';
          end;
          case I.modrm of
            $00..$07, $40..$47, $80..$87, $C0..$C7, $08..$0F, $48..$4F, $88..$8F, $C8..$CF, $10..$17, $50..$57, $90..$97, $D0..$D7,
            $18..$1F, $58..$5F, $98..$9F, $D8..$DF, $20..$27, $60..$67, $A0..$A7, $E0..$E7, $28..$2F, $68..$6F, $A8..$AF, $E8..$EF:
            begin
              tmpStr := DirMem_Reg32(I, AsmText, alig);
            end;
          end;  
        end;
        $01:
        begin
          case I.modrm of
            $00..$07, $40..$47, $80..$87, $C0..$C7: AsmText:= 'SGDT';
            $08..$0F, $48..$4F, $88..$8F, $C8..$CF: AsmText:= 'SIDT';
            $10..$17, $50..$57, $90..$97, $D0..$D7: AsmText:= 'LGDT';
            $18..$1F, $58..$5F, $98..$9F, $D8..$DF: AsmText:= 'LIDT';
          end;
          case I.modrm of
            $00..$07, $40..$47, $80..$87, $C0..$C7, $08..$0F, $48..$4F, $88..$8F, $C8..$CF,
            $10..$17, $50..$57, $90..$97, $D0..$D7, $18..$1F, $58..$5F, $98..$9F, $D8..$DF:
            begin
              alig:= 'FWORD PTR';
              tmpStr := DirMem_Reg32(I, AsmText, alig);
            end;
          end;
          case I.modrm of
            $20..$27, $60..$67, $A0..$A7, $E0..$E7: AsmText:= 'SMSW';
            $30..$37, $70..$77, $B0..$B7, $F0..$F7: AsmText:= 'LMSW';
          end;
          case I.modrm of
            $20..$27, $60..$67, $A0..$A7, $E0..$E7, $30..$37, $70..$77, $B0..$B7, $F0..$F7:
            begin
              alig:= 'WORD PTR';
              tmpStr := DirMem_Reg16(I, AsmText, alig);
            end;
          end;
          case I.modrm of
            $38..$3F, $78..$7F, $B8..$BF, $F8..$FF:
            begin
              alig:= 'BYTE PTR';
              AsmText:= 'INVLPG';
              tmpStr := DirMem_Reg8(I, AsmText, alig);
            end;
          end;
        end;
        $02:
        begin
          AsmText:= 'LAR';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $03:
        begin
          AsmText:= 'LSL';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr:= DirReg32Mem_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $05: tmpStr := 'SYSCALL';
        $06: tmpStr := 'CLTS';
        $07: tmpStr := 'SYSRET';
        $08: tmpStr := 'INVD';
        $09: tmpStr := 'WBINVD';
        $0B: tmpStr := 'UD2';
        $0E: tmpStr := 'FEMMS';
        $30: tmpStr := 'WRMSR';
        $31: tmpStr := 'RDTSC';
        $32: tmpStr := 'RDMSR';
        $33: tmpStr := 'RDPMC';
        $34: tmpStr := 'SYSENTER';
        $35: tmpStr := 'SYSEXIT';
        $40..$4F:
        begin
          AsmText := 'CMOV' + CondStrs[tmpInt-$40];
          alig:= 'DWORD PTR';
          tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
        end;
        $77: tmpStr := 'EMMS';
        $80..$8F:
        begin
          tmpStr := 'J' + CondStrs[tmpInt-$80] + ' ' + IntToHex(A+I.len+Integer(I.rel32),8);
        end;
        $90..$9F:
        begin
          AsmText := 'SET' + CondStrs[tmpInt-$90];
          alig:= 'BYTE PTR';
          tmpStr := DirMem_Reg8(I, AsmText, alig);
        end;
        $A0: tmpStr := 'PUSH FS';
        $A1: tmpStr := 'POP FS';
        $A2: tmpStr := 'CPUID';
        $A3:
        begin
          AsmText:= 'BT';
          alig:= 'DWORD PTR';
          tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
        end;
        $A4:
        begin
          AsmText:= 'SHLD';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig) + ',' + IntToHex(I.imm8,2);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig) + ',' + IntToHex(I.imm8,2);
          end;
        end;
        $A5:
        begin
          AsmText:= 'SHLD';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig) + ',CL';
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig) + ',CL';
          end;
        end;
        $A8: tmpStr := 'PUSH GS';
        $A9: tmpStr := 'POP GS';
        $AA: tmpStr := 'RSM';
        $AB:
        begin
          AsmText:= 'BTS';
          alig:= 'DWORD PTR';
          tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
        end;
       $AC:
        begin
          AsmText:= 'SHRD';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig) + ',' + IntToHex(I.imm8,2);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig) + ',' + IntToHex(I.imm8,2);
          end;
        end;
        $AD:
        begin
          AsmText:= 'SHRD';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig) + ',CL';
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig) + ',CL';
          end;
        end;
        $AF:
        begin
          AsmText:= 'IMUL';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $B0:
        begin
          AsmText:= 'CMPXCHG';
          alig:= 'BYTE PTR';
          tmpStr := DirMemReg8_Reg8Reg8(I, AsmText, alig);
        end;
        $B1:
        begin
          AsmText:= 'CMPXCHG';
          if I.p_66 = $00 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'WORD PTR';
            tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $B2:
        begin
          AsmText:= 'LSS';
          if I.p_66 = $00 then
          begin
            alig:= 'FWORD PTR';
            tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $B3:
        begin
          AsmText:= 'BTR';
          alig:= 'DWORD PTR';
          tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
        end;
        $B4:
        begin
          AsmText:= 'LFS';
          if I.p_66 = $00 then
          begin
            alig:= 'FWORD PTR';
            tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $B5:
        begin
          AsmText:= 'LGS';
          if I.p_66 = $00 then
          begin
            alig:= 'FWORD PTR';
            tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            alig:= 'DWORD PTR';
            tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $B6:
        begin
          asmText:= 'MOVZX';
          alig:= 'BYTE PTR';
          tmpStr := DirReg32Mem_Reg32Reg8(I, AsmText, alig);
        end;
        $B7:
        begin
          asmText:= 'MOVZX';
          alig:= 'WORD PTR';
          tmpStr := DirReg32Mem_Reg32Reg16(I, AsmText, alig);
        end;
        $BA:
        begin
          alig:= 'DWORD PTR';
          case I.modrm of
            $20..$27, $60..$67, $A0..$A7, $E0..$E7: AsmText:= 'BT';
            $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: AsmText:= 'BTS';
            $30..$37, $70..$77, $B0..$B7, $F0..$F7: AsmText:= 'BTR';
            $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: AsmText:= 'BTC';
          end;
          tmpStr := DirMemImm32_Reg32Imm8(I, AsmText, alig);
        end;
        $BB:
        begin
          AsmText:= 'BTC';
          alig:= 'DWORD PTR';
          tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
        end;
        $BC:
        begin
          asmText:= 'BSF';
          alig:= 'DWORD PTR';
          tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
        end;
        $BD:
        begin
          asmText:= 'BSR';
          alig:= 'DWORD PTR';
          tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
        end;
        $BE:
        begin
          asmText:= 'MOVSX';
          alig:= 'BYTE PTR';
          tmpStr := DirReg32Mem_Reg32Reg8(I, AsmText, alig);
        end;
        $BF:
        begin
          asmText:= 'MOVSX';
          alig:= 'WORD PTR';
          tmpStr := DirReg32Mem_Reg32Reg16(I, AsmText, alig);
        end;
        $C0:
        begin
          asmText:= 'XADD';
          alig:= 'BYTE PTR';
          tmpStr := DirMemReg8_Reg8Reg8(I, AsmText, alig);
        end;
        $C1:
        begin
          if I.p_66 = $00 then
          begin
            asmText:= 'XADD';
            alig:= 'DWORD PTR';
            tmpStr := DirMemReg32_Reg32Reg32(I, AsmText, alig);
          end;
          if I.p_66 = $66 then
          begin
            asmText:= 'XADD';
            alig:= 'WORD PTR';
            tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig);
          end;
        end;
        $C7:
        begin
          case I.modrm of
            $08..$0F:
            begin
              asmText:= 'CMPXCHG8B';
              alig:= 'QWORD PTR';
              tmpStr:= DirMem_Reg32(I, AsmText, alig);
            end;
          end;    
        end;
        $C8..$CF:
        begin
          tmpStr := 'BSWAP ';
          tmpStr := tmpStr + Reg32Strs[tmpInt and 7];
        end;
      end;
      if I.p_66 = $66 then
      begin
        case tmpInt of
          $20: tmpStr := 'MOV ' + Reg32Strs[I.modrm_rm] + ',' +  RegCRStrs[I.modrm_reg];
          $21: tmpStr := 'MOV ' + Reg32Strs[I.modrm_rm] + ',' +  RegDRStrs[I.modrm_reg];
          $22: tmpStr := 'MOV ' + RegCRStrs[I.modrm_rm] + ',' +  Reg32Strs[I.modrm_reg];
          $23: tmpStr := 'MOV ' + RegDRStrs[I.modrm_rm] + ',' +  Reg32Strs[I.modrm_reg];
          $24: tmpStr := 'MOV ' + Reg32Strs[I.modrm_rm] + ',' +  RegTRStrs[I.modrm_reg];
          $26: tmpStr := 'MOV ' + RegTRStrs[I.modrm_rm] + ',' +  Reg32Strs[I.modrm_reg];
        end;
      end;
    end;
    $16: tmpStr := 'PUSH SS';
    $17: tmpStr := 'POP SS';
    $1E: tmpStr := 'PUSH DS';
    $1F: tmpStr := 'POP DS';
    $27: tmpStr := 'DAA';
    $2F: tmpStr := 'DAS';
    $37: tmpStr := 'AAA';
    $3F: tmpStr := 'AAS';
    $40..$47:
    begin
      if I.p_66 = $00 then tmpStr := 'INC ' + Reg32Strs[tmpInt - $40];
      if I.p_66 = $66 then tmpStr := 'INC ' + Reg16Strs[tmpInt - $40];
    end;
    $48..$4F:
    begin
      if I.p_66 = $00 then tmpStr := 'DEC ' + Reg32Strs[tmpInt - $48];
      if I.p_66 = $66 then tmpStr := 'DEC ' + Reg16Strs[tmpInt - $48];
    end;
    $50..$57: tmpStr := 'PUSH ' + Reg32Strs[tmpInt - $50];
    $58..$5F: tmpStr := 'POP ' + Reg32Strs[tmpInt - $58];
    $60:
    begin
      if I.p_66 = $00 then tmpStr := 'PUSHAD';
      if I.p_66 = $66 then tmpStr := 'PUSHAW';
    end;
    $61:
    begin
      if I.p_66 = $00 then tmpStr := 'POPAD';
      if I.p_66 = $66 then tmpStr := 'POPAW';
    end;
    $62:
    begin
      if I.p_66 = $00 then
      begin
        asmText:= 'BOUND';
        alig:= 'QDWORD PTR';
        tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
       begin
        asmText:= 'BOUND';
        alig:= 'DWORD PTR';
        tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
      end;
    end;
    $63:
    begin
      asmText:= 'ARPL';
      alig:= 'WORD PTR';
      tmpStr := DirMemReg16_Reg16Reg16(I, AsmText, alig);
    end;
    $68:
    begin
//      if I.p_66 = $00 then tmpStr := 'PUSH '+QuitaCerosIzq(IntToHex(I.imm32,10));
      if I.p_66 = $00 then tmpStr := 'PUSH '+QuitaCerosIzq(IntToHex(I.inm32,10));
      if I.p_66 = $66 then tmpStr := 'PUSH '+IntToHex(I.imm16,8);
    end;
    $69:
    begin
      if I.p_66 = $00 then
      begin
        asmText:= 'IMUL';
        alig:= 'DWORD PTR';
//        tmpStr:= DirReg32Mem_Reg32Reg32(I, AsmText, alig) + ',' + IntToHex(I.Imm32,8);
        tmpStr:= DirReg32Mem_Reg32Reg32(I, AsmText, alig) + ',' + IntToHex(I.Inm32,8);
      end;
      if I.p_66 = $66 then
      begin
        asmText:= 'IMUL';
        alig:= 'WORD PTR';
        tmpStr:= DirReg16Mem_Reg16Reg16(I, AsmText, alig) + ',' + IntToHex(I.Imm16,4);
      end;
    end;
    $6A: tmpStr := 'PUSH '+IntToHex(I.imm8,2);
    $6B:
    begin
      if I.p_66 = $00 then
      begin
        asmText:= 'IMUL';
        alig:= 'DWORD PTR';
        tmpStr:= DirReg32Mem_Reg32Reg32(I, AsmText, alig) + ',' + IntToHex(I.Imm8,2);
      end;
      if I.p_66 = $66 then
      begin
        asmText:= 'IMUL';
        alig:= 'WORD PTR';
        tmpStr:= DirReg16Mem_Reg16Reg16(I, AsmText, alig) + ',' +  IntToHex(I.Imm8,2);
      end;
    end;
    $6C: tmpStr := 'INS BYTE PTR ES:[EDI],DX';
    $6D:
    begin
      if I.p_66 = $00 then tmpStr := 'INS DWORD PTR ES:[EDI],DX';
      if I.p_66 = $66 then tmpStr := 'INS WORD PTR ES:[EDI],DX';
    end;
    $6E: tmpStr := 'OUTS DX,BYTE PTR ES:[EDI]';
    $6F:
    begin
      if I.p_66 = $00 then tmpStr := 'OUTS DX,DWORD PTR ES:[EDI]';
      if I.p_66 = $66 then tmpStr := 'OUTS DX,WORD PTR ES:[EDI]';
    end;
    $70: tmpStr := 'JO ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $71: tmpStr := 'JNO ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $72: tmpStr := 'JB ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $73: tmpStr := 'JNB ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $74: tmpStr := 'JZ ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $75: tmpStr := 'JNZ ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $76: tmpStr := 'JBE ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $77: tmpStr := 'JA ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $78: tmpStr := 'JS ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $79: tmpStr := 'JNS ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $7A: tmpStr := 'JPE ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $7B: tmpStr := 'JPO ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $7C: tmpStr := 'JL ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $7D: tmpStr := 'JGE ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $7E: tmpStr := 'JLE ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $7F: tmpStr := 'JG ' + IntToHex(A+I.len+ShortInt(I.rel8),2);

// ADD; OR; ADC; SBB; AND; SUB; XOR; CMP
//**************************************
    $80, $82:
    begin
      alig:= 'BYTE PTR';
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ADD';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'OR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'ADC';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'SBB';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'AND';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SUB';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'XOR';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'CMP';
      end;
      tmpStr:= DirMemImm8_Reg8Imm8(I, AsmText, alig);
    end;
    $81:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ADD';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'OR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'ADC';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'SBB';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'AND';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SUB';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'XOR';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'CMP';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr:= DirMemImm32_Reg32Imm32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'WORD PTR';
        tmpStr:= DirMemImm16_Reg16Imm16(I, AsmText, alig);
      end;
    end;
    $83:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ADD';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'OR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'ADC';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'SBB';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'AND';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SUB';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'XOR';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'CMP';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr:= DirMemImm32_Reg32Imm8(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'WORD PTR';
        tmpStr:= DirMemImm16_Reg16Imm8(I, AsmText, alig);
      end;
    end;
{    $84:
    begin
      alig:= 'BYTE PTR';
      asmText:= 'TEST';
      tmpStr:= DirMemReg32_Reg32Reg32(I, AsmText, alig);
    end;
 }




// MOV Mem16,Seg
//**************
    $8C:
    begin
      case I.modrm of
        $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$23, $26..$2B, $2E..$33, $36..$3B, $3F:
          tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '],' + RegSegStrs[I.modrm_reg];
        $04, $0C, $14, $1C, $24, $2C, $34, $3C:
        begin
          case I.sib of
            $00..$03, $06..$0C, $0E..$13, $16..$1C, $1E..$1F, $28..$2C, $2E..$33, $36..$3C, $3E..$3F:
                tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'],' + RegSegStrs[I.modrm_reg];
            $04, $14, $34:
                tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'],' + RegSegStrs[I.modrm_reg];
            $05, $0D, $15, $1D, $2D, $35, $3D:
                tmpStr := 'MOV WORD PTR DS:[' +Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + RegSegStrs[I.modrm_reg];    //
            $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6..$E7:
                tmpStr := 'MOV WORD PTR DS:[' +Reg32Strs[I.sib_base] + '],' + Reg32Strs[I.modrm_reg];
            $24, $64, $A4, $E4:
                tmpStr := 'MOV WORD PTR SS:[' +Reg32Strs[I.sib_base] + '],' + RegSegStrs[I.modrm_reg];
            $25, $65, $A5, $E5:
                tmpStr := 'MOV WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + RegSegStrs[I.modrm_reg];
            $40..$43, $46..$4C, $4E..$53, $56..$5C, $5E..$5F, $68..$6C, $6E..$73, $76..$7C, $7E..$7F:
                tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2],' + RegSegStrs[I.modrm_reg];
            $44, $54, $74:
                tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2],' + RegSegStrs[I.modrm_reg];
            $45, $4D, $55, $5D, $6D, $75, $7D:
                tmpStr := 'MOV WORD PTR DS:[' +Reg32Strs[I.sib_index] + '*2+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + RegSegStrs[I.modrm_reg];
            $80..$83, $86..$8C, $8E..$93, $96..$9C, $9E..$9F, $A8..$AC, $AE..$B3, $B6..$BC, $BE..$BF:
                tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4],' + RegSegStrs[I.modrm_reg];
            $84, $94, $B4:
                tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4],' + RegSegStrs[I.modrm_reg];
            $8D, $95, $9D, $AD, $B5, $BD:
                tmpStr := 'MOV WORD PTR DS:[' +Reg32Strs[I.sib_index] + '*8+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + RegSegStrs[I.modrm_reg];
            $C0..$C3, $C6..$CC, $CE..$D3, $D6..$DC, $DE..$DF, $E8..$EC, $EE..$F3, $F6..$FC, $FE..$FF:
                tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8],' + RegSegStrs[I.modrm_reg];
            $C4, $D4, $F4:
                tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8],' + RegSegStrs[I.modrm_reg];
            $C5, $CD, $D5, $DD, $ED, $F5, $FD:
                tmpStr := 'MOV WORD PTR DS:[' +Reg32Strs[I.sib_index] + '*8+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + RegSegStrs[I.modrm_reg];
          end;
        end;
        $05, $0D, $15, $1D, $25, $2D, $35, $3D:
           tmpStr := 'MOV WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + '],' + RegSegStrs[I.modrm_reg];
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$63, $66..$6B, $6E..$73, $76..$7B, $7E..$7F:
          tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' + IntToHex(I.disp8,2) + '],' + RegSegStrs[I.modrm_reg];
        $44, $4C, $54, $5C, $64, $6C, $74, $7C:
          tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.modrm_rm] + '-1],' + RegSegStrs[I.modrm_reg];
        $4D, $5D, $6D, $7D:
          tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.modrm_rm] + '+' + IntToHex(I.disp8,2) + '],' + RegSegStrs[I.modrm_reg];
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$A3, $A6..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
          tmpStr := 'MOV WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' +QuitaCerosIzq(IntToHex(I.disp32,10))+ '],' + RegSegStrs[I.modrm_reg];
        $84, $8C, $94, $9C, $A4, $AC, $B4, $BC:
          tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.modrm_rm] + '+' +QuitaCerosIzq(IntToHex(I.disp32,10))+ '],' + RegSegStrs[I.modrm_reg];
        $8D, $9D, $AD, $BD:
          tmpStr := 'MOV WORD PTR SS:[' + Reg32Strs[I.modrm_rm] + '+' +QuitaCerosIzq(IntToHex(I.disp32,10))+ '],' + RegSegStrs[I.modrm_reg];
        $C0..$FF: tmpStr := 'MOV ' + Reg16Strs[I.modrm_rm] + ',' + RegSegStrs[I.modrm_reg];
       end;
    end;
    $8D:
    begin
      AsmText:= 'LEA';
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'DWORD PTR';
        tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
      end;
    end;
// MOV Seg,Mem16
//**************
    $8E:
    begin
      case I.modrm of
         $00..$03, $06..$0B, $0E..$13, $16..$1B, $1E..$23, $26..$2B, $2E..$33, $36..$3B, $3F:
           tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] +']';
        $04, $0C, $14, $1C, $24, $2C, $34, $3C:
        begin
          case I.sib of
            $00..$03, $06..$0C, $0E..$13, $16..$1C, $1E..$1F, $28..$2C, $2E..$33, $36..$3C, $3E..$3F:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +']';
            $04, $14, $34:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +']';
            $05, $0D, $15, $1D, $2D, $35, $3D:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' +Reg32Strs[I.sib_index] + '+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';    //
            $20..$23, $26..$27, $60..$63, $66..$67, $A0..$A3, $A6..$A7, $E0..$E3, $E6..$E7:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' +Reg32Strs[I.sib_base] + ']';
            $24, $64, $A4, $E4:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' +Reg32Strs[I.sib_base] + ']';
            $25, $65, $A5, $E5:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
            $40..$43, $46..$4C, $4E..$53, $56..$5C, $5E..$5F, $68..$6C, $6E..$73, $76..$7C, $7E..$7F:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2]';
            $44, $54, $74:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*2]';
            $45, $4D, $55, $5D, $6D, $75, $7D:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' +Reg32Strs[I.sib_index] + '*2+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
            $80..$83, $86..$8C, $8E..$93, $96..$9C, $9E..$9F, $A8..$AC, $AE..$B3, $B6..$BC, $BE..$BF:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4]';
            $84, $94, $B4:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*4]';
            $8D, $95, $9D, $AD, $B5, $BD:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' +Reg32Strs[I.sib_index] + '*8+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
            $C0..$C3, $C6..$CC, $CE..$D3, $D6..$DC, $DE..$DF, $E8..$EC, $EE..$F3, $F6..$FC, $FE..$FF:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8]';
            $C4, $D4, $F4:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.sib_base] + '+' + Reg32Strs[I.sib_index] +'*8]';
            $C5, $CD, $D5, $DD, $ED, $F5, $FD:
                tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' +Reg32Strs[I.sib_index] + '*8+' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
          end;
        end;
        $05, $0D, $15, $1D, $25, $2D, $35, $3D:
           tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.disp32,10)) + ']';
        $40..$43, $46..$4B, $4E..$53, $56..$5B, $5E..$63, $66..$6B, $6E..$73, $76..$7B, $7E..$7F:
          tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' + IntToHex(I.disp8,2) + ']';
        $44, $4C, $54, $5C, $64, $6C, $74, $7C:
          tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '-1],' + Reg32Strs[I.modrm_reg];
        $4D, $5D, $6D, $7D:
          tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' + IntToHex(I.disp8,2) + ']';
        $80..$83, $86..$8B, $8E..$93, $96..$9B, $9E..$A3, $A6..$AB, $AE..$B3, $B6..$BB, $BE..$BF:
          tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' +QuitaCerosIzq(IntToHex(I.disp32,10))+ ']';
        $84, $8C, $94, $9C, $A4, $AC, $B4, $BC:
          tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' +QuitaCerosIzq(IntToHex(I.disp32,10))+ ']';
        $8D, $9D, $AD, $BD:
          tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',WORD PTR DS:[' + Reg32Strs[I.modrm_rm] + '+' +QuitaCerosIzq(IntToHex(I.disp32,10))+ ']';
// MOV Seg,Reg16
//*************
        $C0..$FF: tmpStr := 'MOV ' + RegSegStrs[I.modrm_reg] + ',' + Reg16Strs[I.modrm_reg];
      end;
    end;

    $8F:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: AsmText:= 'POP';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr := DirMem_Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'WORD PTR';
        tmpStr := DirMem_Reg16(I, AsmText, alig);
      end;
    end;
//************
    $90: tmpStr := 'NOP';
    $91..$97:
    begin
      if I.p_66 = $00 then tmpStr := 'XCHG EAX,' + Reg32Strs[tmpInt - $90];
      if I.p_66 = $66 then tmpStr := 'XCHG AX,' + Reg16Strs[tmpInt - $90];
    end;
    $98:
    begin
      if I.p_66 = $00 then tmpStr := 'CWDE';
      if I.p_66 = $66 then tmpStr := 'CBW';
    end;
    $99:
    begin
      if I.p_66 = $00 then tmpStr := 'CDQ';
      if I.p_66 = $66 then tmpStr := 'CWD';
    end;
//    $9A: tmpStr := 'CALL FAR ' + IntToHex(I.Imm16,4) + ':' + QuitaCerosIzq(IntToHex(I.imm32,10));
    $9A: tmpStr := 'CALL FAR ' + IntToHex(I.Imm16,4) + ':' + QuitaCerosIzq(IntToHex(I.inm32,10));
    $9B: tmpStr := 'WAIT';
    $9C:
    begin
      if I.p_66 = $00 then tmpStr := 'PUSHFD';
      if I.p_66 = $66 then tmpStr := 'PUSHFW';
    end;
    $9D:
    begin
      if I.p_66 = $00 then tmpStr := 'POPFD';
      if I.p_66 = $66 then tmpStr := 'POPFW';
    end;
    $9E: tmpStr := 'SAHF';
    $9F: tmpStr := 'LAHF';

// MOV Acc,MemOfs
//***************
//    $A0: tmpStr := 'MOV AL, BYTE PTR DS:[' + QuitaCerosIzq(IntToHex(I.imm32,10))+']';
    $A0: tmpStr := 'MOV AL, BYTE PTR DS:[' + QuitaCerosIzq(IntToHex(I.inm32,10))+']';
    $A1:
    begin
//      if I.p_66 = $00 then tmpStr := 'MOV EAX, DWORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.imm32,10))+']';
      if I.p_66 = $00 then tmpStr := 'MOV EAX, DWORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.inm32,10))+']';
//      if I.p_66 = $66 then tmpStr := 'MOV AX, WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.imm32,10))+']';
      if I.p_66 = $66 then tmpStr := 'MOV AX, WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.inm32,10))+']';
    end;

// MOV MemOfs,Acc
//***************
//    $A2: tmpStr := 'MOV BYTE PTR DS:[' + QuitaCerosIzq(IntToHex(I.imm32,10))+'],AL';
    $A2: tmpStr := 'MOV BYTE PTR DS:[' + QuitaCerosIzq(IntToHex(I.inm32,10))+'],AL';
    $A3:
    begin
//      if I.p_66 = $00 then tmpStr := 'MOV DWORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.imm32,10))+'],EAX';
      if I.p_66 = $00 then tmpStr := 'MOV DWORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.inm32,10))+'],EAX';
//      if I.p_66 = $66 then tmpStr := 'MOV WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.imm32,10))+'],AX';
      if I.p_66 = $66 then tmpStr := 'MOV WORD PTR DS:[' + QuitaCerosIzq(IntToHex(I.inm32,10))+'],AX';
    end;

// MOVSB, MOVSW, MOVSD
//********************
    $A4: tmpStr := 'MOVS BYTE PTR ES:[EDI],BYTE PTR DS:[ESI]';
    $A5:
    begin
      if I.p_66 = $00 then tmpStr := 'MOVS DWORD PTR ES:[EDI],DWORD PTR DS:[ESI]';
      if I.p_66 = $66 then tmpStr := 'MOVS WORD PTR ES:[EDI],WORD PTR DS:[ESI]';
    end;
    $A6: tmpStr := 'CMPS BYTE PTR DS:[ESI],BYTE PTR ES:[EDI]';
    $A7:
    begin
      if I.p_66 = $00 then tmpStr := 'CMPS DWORD PTR DS:[ESI],BYTE PTR ES:[EDI]';
      if I.p_66 = $66 then tmpStr := 'CMPS WORD PTR DS:[ESI],BYTE PTR ES:[EDI]';
    end;
    $AA: tmpStr := 'STOS BYTE PTR ES:[EDI]';
    $AB:
    begin
      if I.p_66 = $00 then tmpStr := 'STOS DWORD PTR ES:[EDI]';
      if I.p_66 = $66 then tmpStr := 'STOS WORD PTR ES:[EDI]';

    end;
    $AC: tmpStr := 'LODS BYTE PTR DS:[ESI]';
    $AD:
    begin
      if I.p_66 = $00 then tmpStr := 'LODS DWORD PTR DS:[ESI]';
      if I.p_66 = $66 then tmpStr := 'LODS WORD PTR DS:[ESI]';
    end;
    $AE: tmpStr := 'SCAS BYTE PTR ES:[EDI]';
    $AF:
    begin
      if I.p_66 = $00 then tmpStr := 'SCAS DWORD PTR ES:[EDI]';
      if I.p_66 = $66 then tmpStr := 'SCAS WORD PTR ES:[EDI]';
    end;
    $B0..$B7: tmpStr := 'MOV ' + Reg8Strs[tmpInt and $7] + ',' + IntToHex(I.imm8,2);
//    $B8..$BF: tmpStr := 'MOV ' + Reg32Strs[(tmpInt and $F)-8] + ',' + QuitaCerosIzq(IntToHex(I.imm32,10));
    $B8..$BF: tmpStr := 'MOV ' + Reg32Strs[(tmpInt and $F)-8] + ',' + QuitaCerosIzq(IntToHex(I.inm32,10));
    $C1:
      begin
      alig:= 'DWORD PTR';
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ROL';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'ROR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'RCL';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'RCR';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'SHL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SHR';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'SAL';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'SAR';
      end;
      tmpStr:= DirMemImm32_Reg32Imm8(I, AsmText, alig);
    end;
    $C2: tmpStr := 'RETN '+ IntToHex(I.imm16,4);
    $C3: tmpStr := 'RET';
    $C4:
    begin
      AsmText:= 'LES';
      if I.p_66 = $00 then
      begin
        alig:= 'FWORD PTR';
        tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'DWORD PTR';
        tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
      end;
    end;
    $C5:
    begin
      AsmText:= 'LDS';
      if I.p_66 = $00 then
      begin
        alig:= 'FWORD PTR';
        tmpStr := DirReg32Mem_Reg32Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig:= 'DWORD PTR';
        tmpStr := DirReg16Mem_Reg16Reg16(I, AsmText, alig);
      end;
    end;

// MOV Mem,Imm8 MOV Reg8,Imm8
//***************************
    $C6:
    begin
      alig:= 'BYTE PTR';
      asmText:= 'MOV';
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $D0..$FF: tmpStr := DirMemImm8_Reg8Imm8(I, AsmText, alig);
      end;
    end;
// MOV Mem,Imm32 MOV Reg32,Imm32
//******************************
    $C7:
    begin
      if I.p_66 = $00 then
      begin
        asmText:= 'MOV';
        alig:= 'DWORD PTR';
      end;
      if I.p_66 = $66 then
      begin
        asmText:= 'MOV';
        alig:= 'WORD PTR';
      end;
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $D0..$FF: tmpStr:= DirMemImm32_Reg32Imm32(I, AsmText, alig);
      end;
    end;
    $C8: tmpStr:= 'ENTER ' + IntToHex(I.Imm16,4) + ',' + IntToHex(I.Imm8,2);
    $C9: tmpStr := 'LEAVE';
    $CA: tmpStr := 'RETF '+ IntToHex(I.imm16,4);
    $CB: tmpStr := 'RETF';
    $CC: tmpStr := 'INT3';
    $CD: tmpStr := 'INT '+IntToHex(I.imm8,2);
    $CE: tmpStr := 'INTO';
    $CF: tmpStr := 'IRETD';
    $D0:
    begin
      alig:= 'BYTE PTR';
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ROL';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'ROR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'RCL';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'RCR';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'SHL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SHR';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'SAL';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'SAR';
      end;
      tmpStr:= DirMem_Reg8(I, AsmText, alig) + ',1';
    end;
    $D1:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ROL';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'ROR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'RCL';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'RCR';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'SHL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SHR';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'SAL';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'SAR';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr:= DirMem_Reg32(I, AsmText, alig) + ',1';
      end;
      if I.p_66 = $66 then alig:= 'WORD PTR';
      begin
        alig:= 'WORD PTR';
        tmpStr:= DirMem_Reg16(I, AsmText, alig) + ',1';
      end;
    end;
    $D2:
    begin
      alig:= 'BYTE PTR';
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ROL';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'ROR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'RCL';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'RCR';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'SHL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SHR';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'SAL';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'SAR';
      end;
      tmpStr:= DirMem_Reg8(I, AsmText, alig) + ',CL';
    end;
    $D3:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'ROL';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'ROR';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'RCL';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'RCR';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'SHL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'SHR';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'SAL';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'SAR';
      end;
      if I.p_66 = $00 then
      begin
        alig:= 'DWORD PTR';
        tmpStr:= DirMem_Reg32(I, AsmText, alig) + ',CL';
      end;
      if I.p_66 = $66 then alig:= 'WORD PTR';
      begin
        alig:= 'WORD PTR';
        tmpStr:= DirMem_Reg16(I, AsmText, alig) + ',CL';
      end;
    end;
    $D4:
    begin
      case I.imm8 of
        $0A: tmpStr := 'AAM';
        else
          tmpStr := 'AAM ' + IntToHex(I.imm8,2);
      end;
    end;
    $D5:
    begin
      case I.imm8 of
        $0A: tmpStr := 'AAD';
        else
          tmpStr := 'AAD ' + IntToHex(I.imm8,2);
      end;
    end;
    $D6: tmpStr := 'SALC';
    $D7: tmpStr := 'XLAT BYTE PTR DS:[EBX+AL]';
    $D8..$DF:
    begin
      tmpStr := 'Instruccion coprocesador. L = '+IntToStr(I.Len);
    end;
    $E0:
      begin
      if I.p_67 = $00 then tmpStr := 'LOOPDNE ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
      if I.p_66 = $67 then tmpStr := 'LOOPWNE ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
    end;
    $E1:
      begin
      if I.p_67 = $00 then tmpStr := 'LOOPDE ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
      if I.p_67 = $67 then tmpStr := 'LOOPWE ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
    end;
    $E2:
      begin
      if I.p_67 = $00 then tmpStr := 'LOOPD ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
      if I.p_67 = $67 then tmpStr := 'LOOPW ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
    end;
    $E3:
    begin
      if I.p_67 = $00 then tmpStr := 'JECXZ SHORT ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
      if I.p_67 = $67 then tmpStr := 'JCXZ SHORT ' + IntToHex(A+I.len+ShortInt(I.rel8),8);
    end;
    $E4: tmpStr := 'IN AL,' + IntToHex(I.Imm8,2);
    $E5: tmpStr := 'IN EAX,' + IntToHex(I.Imm8,2);
    $E6: tmpStr := 'OUT AL,' + IntToHex(I.Imm8,2);
    $E7: tmpStr := 'OUT EAX,' + IntToHex(I.Imm8,2);
    $E8: tmpStr := 'CALL ' + IntToHex(A+I.len+Integer(I.rel32),8);
    $E9: tmpStr := 'JMP '+ IntToHex(A+I.len+Integer(I.rel32),8);
//    $EA: tmpStr := 'JMP FAR '+IntToHex(I.imm16,4)+':'+QuitaCerosIzq(IntToHex(I.imm32,10));
    $EA: tmpStr := 'JMP FAR '+IntToHex(I.imm16,4)+':'+QuitaCerosIzq(IntToHex(I.inm32,10));
    $EB: tmpStr := 'JMP ' + IntToHex(A+I.len+ShortInt(I.rel8),2);
    $EC: tmpStr := 'IN AL,DX';
    $ED: tmpStr := 'IN EAX,DX';
    $EE: tmpStr := 'OUT DX,AL';
    $EF:
    begin
      if I.p_67 = $00 then tmpStr := 'OUT DX,EAX';
      if I.p_66 = $67 then tmpStr := 'OUT DX,AX';
    end;  
    $F1: tmpStr := 'INT1';
    $F4: tmpStr := 'HLT';
    $F5: tmpStr := 'CMC';

// NOT, NEG, MUL, IMUL, DIV, IDIV Mem8; NOT, NEG, MUL, IMUL, DIV, IDIV Reg8;
//*************************************************************************
    $F6:
    begin
      alig:= 'BYTE PTR';
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'TEST';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'NOT';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'NEG';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'MUL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'IMUL';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'DIV';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'IDIV';
      end;
      tmpStr:= DirMem_Reg8(I, AsmText, alig);
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: tmpStr:= tmpStr + ',' + IntToHex(I.imm8,2);
      end;  
    end;

// NOT, NEG, MUL, IMUL, DIV, IDIV Mem32; NOT, NEG, MUL, IMUL, DIV, IDIV Reg32;
//***************************************************************************
    $F7:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'TEST';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'NOT';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'NEG';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'MUL';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'IMUL';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'DIV';
        $38..$3F, $78..$7F, $B8..$BF, $F8..$FF: asmText:= 'IDIV';
      end;
      if I.p_66 = $00 then
      begin
        alig := 'DWORD';
        tmpStr:= DirMem_Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig := 'WORD';
        tmpStr:= DirMem_Reg16(I, AsmText, alig);
      end;
    end;
    $F8: tmpStr := 'CLC';
    $F9: tmpStr := 'STC';
    $FA: tmpStr := 'CLI';
    $FB: tmpStr := 'STI';
    $FC: tmpStr := 'CLD';
    $FD: tmpStr := 'STD';

// INC, DEC Mem; INC, DEC Reg8
//****************************
    $FE:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'INC';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'DEC';
      end;
      alig:= 'BYTE PTR';
      tmpStr:= DirMem_Reg8(I, AsmText, alig);
    end;
// INC, DEC Mem; INC, DEC Reg32
//*****************************
    $FF:
    begin
      case I.modrm of
        $00..$07, $40..$47, $80..$87, $C0..$C7: asmText:= 'INC';
        $08..$0F, $48..$4F, $88..$8F, $C8..$CF: asmText:= 'DEC';
        $10..$17, $50..$57, $90..$97, $D0..$D7: asmText:= 'CALL';
        $18..$1F, $58..$5F, $98..$9F, $D8..$DF: asmText:= 'CALL FAR';
        $20..$27, $60..$67, $A0..$A7, $E0..$E7: asmText:= 'JMP';
        $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: asmText:= 'JMP FAR';
        $30..$37, $70..$77, $B0..$B7, $F0..$F7: asmText:= 'PUSH';
      end;
      if I.p_66 = $00 then
      begin
        alig := 'DWORD PTR';
        case I.modrm of
          $18..$1F, $58..$5F, $98..$9F, $D8..$DF, $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: alig := 'FWORD PTR';
{          $00..$07, $40..$47, $80..$87, $C0..$C7, $08..$0F, $48..$4F, $88..$8F, $C8..$CF,
          $10..$17, $50..$57, $90..$97, $D0..$D7, $20..$27, $60..$67, $A0..$A7, $E0..$E7,
          $30..$37, $70..$77, $B0..$B7, $F0..$F7: alig := 'DWORD PTR';
 //         else alig := 'DWORD PTR';}
        end;
        tmpStr:= DirMem_Reg32(I, AsmText, alig);
      end;
      if I.p_66 = $66 then
      begin
        alig := 'WORD PTR';
        case I.modrm of
          $18..$1F, $58..$5F, $98..$9F, $D8..$DF, $28..$2F, $68..$6F, $A8..$AF, $E8..$EF: alig := 'DWORD PTR';
        end;
        tmpStr:= DirMem_Reg16(I, AsmText, alig);
      end;
    end;

  end;


// Prefix
//*******  
  if I.p_lock = $F0 then tmpStr:= 'LOCK ' + tmpStr;
  if I.p_rep =  $F2 then tmpStr:= 'REPNE ' + tmpStr;   
  if I.p_rep =  $F3 then tmpStr:= 'REP ' + tmpStr;
  case I.p_seg of
    $2E: tmpStr:= ReemplazaSeg(tmpStr,'C');
    $26: tmpStr:= ReemplazaSeg(tmpStr,'E');
    $36: tmpStr:= ReemplazaSeg(tmpStr,'S');
    $64: tmpStr:= ReemplazaSeg(tmpStr,'F');
    $65: tmpStr:= ReemplazaSeg(tmpStr,'G');
  end;
  Result:= tmpStr;
end;



end.
