unit Find;

//**************************************************
// Gracias Arapumk por pasarme la parte de cdigo
//**************************************************

interface

uses
  Windows, Sysutils, Classes;

type
  TBMJumpTable = array[0..255] of Integer;
  TFastPosProc = function (const aSource, aFind: Pointer; const aSourceLen, aFindLen: Integer; var JumpTable: TBMJumpTable): Pointer;
  TFastPosIndexProc = function (const aSourceString, aFindString: string; const aSourceLen, aFindLen, StartPos: Integer; var JumpTable: TBMJumpTable): Integer;
  TFastTagReplaceProc = procedure (var Tag: string; const UserData: Integer);

//Boyer-Moore routines
// internas
procedure MakeBMTable(Buffer: PChar; BufferLen: Integer; var JumpTable: TBMJumpTable);
function BMPos(const aSource, aFind: Pointer; const aSourceLen, aFindLen: Integer; var JumpTable: TBMJumpTable): Pointer;
// publicas

function FastStrPos(const aSourceString, aFindString : PChar; const LenSource, LenFind : Integer) : Integer;

implementation

//MakeBMJumpTable takes a FindString and makes a JumpTable
procedure MakeBMTable(Buffer: PChar; BufferLen: Integer; var JumpTable: TBMJumpTable);
begin
  if BufferLen = 0 then raise Exception.Create('BufferLen is 0');
  asm
        push    EDI
        push    ESI

        mov     EDI, JumpTable
        mov     EAX, BufferLen
        mov     ECX, $100
        REPNE   STOSD

        mov     ECX, BufferLen
        mov     EDI, JumpTable
        mov     ESI, Buffer
        dec     ECX
        xor     EAX, EAX
@@loop:
        mov     AL, [ESI]
        lea     ESI, ESI + 1
        mov     [EDI + EAX * 4], ECX
        dec     ECX
        jg      @@loop

        pop     ESI
        pop     EDI
  end;
end;


function BMPos(const aSource, aFind: Pointer; const aSourceLen, aFindLen: Integer; var JumpTable: TBMJumpTable): Pointer;
var
  LastPos: Pointer;
begin
  LastPos := Pointer(Integer(aSource) + aSourceLen - 1);
  asm
        push    ESI
        push    EDI
        push    EBX

        mov     EAX, aFindLen
        mov     ESI, aSource
        lea     ESI, ESI + EAX - 1
        std
        mov     EBX, JumpTable

@@comparetext:
        cmp     ESI, LastPos
        jg      @@NotFound
        mov     EAX, aFindLen
        mov     EDI, aFind
        mov     ECX, EAX
        push    ESI //Remember where we are
        lea     EDI, EDI + EAX - 1
        xor     EAX, EAX
@@CompareNext:
        mov     al, [ESI]
        cmp     al, [EDI]
        jne     @@LookAhead
        lea     ESI, ESI - 1
        lea     EDI, EDI - 1
        dec     ECX
        jz      @@Found
        jmp     @@CompareNext

@@LookAhead:
        //Look up the char in our Jump Table
        pop     ESI
        mov     al, [ESI]
        mov     EAX, [EBX + EAX * 4]
        lea     ESI, ESI + EAX
        jmp     @@CompareText

@@NotFound:
        mov     Result, 0
        jmp     @@TheEnd
@@Found:
        pop     EDI //We are just popping, we don't need the value
        inc     ESI
        mov     Result, ESI
@@TheEnd:
        cld
        pop     EBX
        pop     EDI
        pop     ESI
  end;
end;


// Busca un Pchar dentro de un Buffer si la busqueda tiene exito devuelve la posicin
// de la primera ocurrencia si no devuelve -1
// la funcion tiene 4 parametros
// aSourceString -> sitio donde buscar
// aFindString   -> string a buscar;
// LenSource     -> longitud del buffer donde buscar
// LenFind       -> longitud de la String a buscar


function FastStrPos(const aSourceString, aFindString : PChar; const LenSource, LenFind : Integer) : Integer;
var
  JumpTable: TBMJumpTable;
  StartPos  : integer;
begin
  Result := -1;
  StartPos := 1;
  if LenFind < 1 then  exit;
  if LenFind > LenSource then exit;


  MakeBMTable(aFindString, LenFind, JumpTable);
  Result := Integer(BMPos(aSourceString + (StartPos - 1), aFindString,LenSource - (StartPos-1), LenFind, JumpTable));
  if Result > 0 then
    Result := Result - Integer(@aSourceString[1]) +1
  else Result := -1;
end;

end.
